package org.reproducer;

import java.net.URL;

import javax.annotation.PostConstruct;
import javax.ejb.Singleton;
import javax.ejb.Startup;

import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.process.ProcessInstance;
import org.kie.services.client.api.RemoteJmsRuntimeEngineFactory;
import org.kie.services.client.api.RemoteRestRuntimeEngineFactory;
import org.kie.services.client.api.RemoteRuntimeEngineFactory;
import org.kie.services.client.api.builder.RemoteJmsRuntimeEngineFactoryBuilder;
import org.kie.services.client.api.builder.RemoteRestRuntimeEngineFactoryBuilder;
import org.kie.services.client.api.builder.RemoteRuntimeEngineFactoryBuilder;
import org.kie.services.client.api.command.RemoteRuntimeEngine;

@Startup
@Singleton
public class ReproducerBean {

    @PostConstruct
    public void test() throws Exception {
        testJms();
        testRest();
    }

    private void testJms() {
        System.out.println("******** testJms(): start **********");
        try {
            RemoteJmsRuntimeEngineFactoryBuilder builder = RemoteJmsRuntimeEngineFactory.newBuilder();
            builder.addJbossServerUrl(new URL("http://localhost:8080"));
            //builder.addHostName("");
            testCommon(builder);
        } catch (Exception e) {
            System.err.println(e);
            //e.printStackTrace();
        }
        System.out.println("******** testJms(): finish **********");
    }

    private void testRest() {
        System.out.println("******** testRest(): start **********");
        try {
            RemoteRestRuntimeEngineFactoryBuilder builder = RemoteRestRuntimeEngineFactory.newBuilder();
            builder.addUrl(new URL("http://localhost:8080/kie-wb/"));
            testCommon(builder);
        } catch (Exception e) {
            System.err.println(e);
            //e.printStackTrace();
        }
        System.out.println("******** testRest(): finish **********");
    }

    private <B extends RemoteRuntimeEngineFactoryBuilder<B,? extends RemoteRuntimeEngineFactory>> void testCommon(B builder) throws Exception {
        builder.addDeploymentId("mortgages:mortgages:0.0.1");
        builder.addUserName("admin");
        builder.addPassword("the-frog-1");
        builder.addTimeout(2);
        RemoteRuntimeEngineFactory factory = builder.build();
        RemoteRuntimeEngine engine = factory.newRuntimeEngine();
        KieSession session = engine.getKieSession();
        ProcessInstance processInstance = session.startProcess("org.mortgages.Hello");
        if (processInstance != null) {
            long pid = processInstance.getId();
            System.out.println("Success! (pid: " + pid + ")");
        } else {
            System.err.println("Error! (null processInstance)");
        }
    }

    public static void main(String... args) {
        new ReproducerBean().testRest();
    }

}
